@@ -50,25 +50,31 @@ class HttpRequest {
   HttpRequest() {}
   virtual ~HttpRequest() {}
 
-  virtual Status Init() = 0;
-
   /// Sets the request URI.
-  virtual Status SetUri(const string& uri) = 0;
+  virtual void SetUri(const string& uri) = 0;
 
   /// \brief Sets the Range header.
   ///
   /// Used for random seeks, for example "0-999" returns the first 1000 bytes
   /// (note that the right border is included).
-  virtual Status SetRange(uint64 start, uint64 end) = 0;
+  virtual void SetRange(uint64 start, uint64 end) = 0;
 
   /// Sets a request header.
-  virtual Status AddHeader(const string& name, const string& value) = 0;
+  virtual void AddHeader(const string& name, const string& value) = 0;
+
+  /// Sets a DNS resolve mapping (to skip DNS resolution).
+  ///
+  /// Note: because GCS is available over HTTPS, we cannot replace the hostname
+  /// in the URI with an IP address, as that will cause the certificate check
+  /// to fail.
+  virtual void AddResolveOverride(const string& hostname, int64 port,
+                                  const string& ip_addr) = 0;
 
   /// Sets the 'Authorization' header to the value of 'Bearer ' + auth_token.
-  virtual Status AddAuthBearerHeader(const string& auth_token) = 0;
+  virtual void AddAuthBearerHeader(const string& auth_token) = 0;
 
   /// Makes the request a DELETE request.
-  virtual Status SetDeleteRequest() = 0;
+  virtual void SetDeleteRequest() = 0;
 
   /// \brief Makes the request a PUT request.
   ///
@@ -77,21 +83,35 @@ class HttpRequest {
   virtual Status SetPutFromFile(const string& body_filepath, size_t offset) = 0;
 
   /// Makes the request a PUT request with an empty body.
-  virtual Status SetPutEmptyBody() = 0;
+  virtual void SetPutEmptyBody() = 0;
 
   /// \brief Makes the request a POST request.
   ///
   /// The request body will be taken from the specified buffer.
-  virtual Status SetPostFromBuffer(const char* buffer, size_t size) = 0;
+  virtual void SetPostFromBuffer(const char* buffer, size_t size) = 0;
 
   /// Makes the request a POST request with an empty body.
-  virtual Status SetPostEmptyBody() = 0;
+  virtual void SetPostEmptyBody() = 0;
 
   /// \brief Specifies the buffer for receiving the response body.
   ///
   /// Size of out_buffer after an access will be exactly the number of bytes
   /// read. Existing content of the vector will be cleared.
-  virtual Status SetResultBuffer(std::vector<char>* out_buffer) = 0;
+  virtual void SetResultBuffer(std::vector<char>* out_buffer) = 0;
+
+  /// \brief Specifies the buffer for receiving the response body.
+  ///
+  /// This method should be used when a caller knows the upper bound of the
+  /// size of the response data.  The caller provides a pre-allocated buffer
+  /// and its size. After the Send() method is called, the
+  /// GetResultBufferDirectBytesTransferred() method may be used to learn to the
+  /// number of bytes that were transferred using this method.
+  virtual void SetResultBufferDirect(char* buffer, size_t size) = 0;
+
+  /// \brief Returns the number of bytes transferred, when using
+  /// SetResultBufferDirect(). This method may only be used when using
+  /// SetResultBufferDirect().
+  virtual size_t GetResultBufferDirectBytesTransferred() = 0;
 
   /// \brief Returns the response headers of a completed request.
   ///
@@ -110,6 +130,16 @@ class HttpRequest {
   // Url encodes str and returns a new string.
   virtual string EscapeString(const string& str) = 0;
 
+  /// \brief Set timeouts for this request.
+  ///
+  /// The connection parameter controls how long we should wait for the
+  /// connection to be established. The inactivity parameter controls how long
+  /// we should wait between additional responses from the server. Finally the
+  /// total parameter controls the maximum total connection time to prevent
+  /// hanging indefinitely.
+  virtual void SetTimeouts(uint32 connection, uint32 inactivity,
+                           uint32 total) = 0;
+
   TF_DISALLOW_COPY_AND_ASSIGN(HttpRequest);
 };
 
