@@ -32,6 +32,7 @@
 @@device
 @@container
 @@name_scope
+@@colocate_with
 @@control_dependencies
 @@convert_to_tensor
 @@convert_to_tensor_or_indexed_slices
@@ -85,6 +86,7 @@ from tensorflow.python.framework.ops import device
 from tensorflow.python.framework.ops import container
 from tensorflow.python.framework.ops import name_scope
 from tensorflow.python.framework.ops import op_scope
+from tensorflow.python.framework.ops import colocate_with
 from tensorflow.python.framework.ops import control_dependencies
 from tensorflow.python.framework.ops import get_default_graph
 from tensorflow.python.framework.ops import reset_default_graph
@@ -97,7 +99,6 @@ from tensorflow.python.framework.ops import convert_to_tensor_or_indexed_slices
 from tensorflow.python.framework.random_seed import get_seed
 from tensorflow.python.framework.random_seed import set_random_seed
 from tensorflow.python.framework.sparse_tensor import convert_to_tensor_or_sparse_tensor
-from tensorflow.python.framework.subscribe import subscribe
 from tensorflow.python.framework.importer import import_graph_def
 
 # Utilities for working with Tensors
