@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
 """Operations often used for initializing tensors.
 
 All variable initializers returned by functions in this file should have the
@@ -35,13 +34,20 @@ from __future__ import print_function
 
 import math
 
+import numpy as np
+
 from tensorflow.python.framework import constant_op
 from tensorflow.python.framework import dtypes
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import linalg_ops
+from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import random_ops
+from tensorflow.python.ops import random_ops
+from tensorflow.python.util.deprecation import deprecated
+from tensorflow.python.util.tf_export import tf_export
 
 
+@tf_export("keras.initializers.Initializer")
 class Initializer(object):
   """Initializer base class: all initializers inherit from this class.
   """
@@ -49,32 +55,71 @@ class Initializer(object):
   def __call__(self, shape, dtype=None, partition_info=None):
     raise NotImplementedError
 
+  def get_config(self):
+    """Returns the configuration of the initializer as a JSON-serializable dict.
 
+    Returns:
+      A JSON-serializable Python dict.
+    """
+    return {}
+
+  @classmethod
+  def from_config(cls, config):
+    """Instantiates an initializer from a configuration dictionary.
+
+    Example:
+
+    ```python
+    initializer = RandomUniform(-1, 1)
+    config = initializer.get_config()
+    initializer = RandomUniform.from_config(config)
+    ```
+
+    Args:
+      config: A Python dictionary.
+        It will typically be the output of `get_config`.
+
+    Returns:
+      An Initializer instance.
+    """
+    return cls(**config)
+
+
+@tf_export("keras.initializers.Zeros", "initializers.zeros",
+           "zeros_initializer")
 class Zeros(Initializer):
   """Initializer that generates tensors initialized to 0."""
 
   def __init__(self, dtype=dtypes.float32):
-    self.dtype = dtype
+    self.dtype = dtypes.as_dtype(dtype)
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
       dtype = self.dtype
-    return constant_op.constant(False if dtype is dtypes.bool else 0,
-                                dtype=dtype, shape=shape)
+    return array_ops.zeros(shape, dtype)
+
+  def get_config(self):
+    return {"dtype": self.dtype.name}
 
 
+@tf_export("keras.initializers.Ones", "initializers.ones", "ones_initializer")
 class Ones(Initializer):
   """Initializer that generates tensors initialized to 1."""
 
   def __init__(self, dtype=dtypes.float32):
-    self.dtype = dtype
+    self.dtype = dtypes.as_dtype(dtype)
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
       dtype = self.dtype
-    return constant_op.constant(1, dtype=dtype, shape=shape)
+    return array_ops.ones(shape, dtype)
+
+  def get_config(self):
+    return {"dtype": self.dtype.name}
 
 
+@tf_export("keras.initializers.Constant", "initializers.constant",
+           "constant_initializer")
 class Constant(Initializer):
   """Initializer that generates tensors with constant values.
 
@@ -92,14 +137,17 @@ class Constant(Initializer):
   tensor shape, the initializer will raise a `ValueError`.
 
   Args:
-    value: A Python scalar, list of values, or a N-dimensional numpy array. All
-      elements of the initialized variable will be set to the corresponding
-      value in the `value` argument.
+    value: A Python scalar, list or tuple of values, or a N-dimensional numpy
+      array. All elements of the initialized variable will be set to the
+      corresponding value in the `value` argument.
     dtype: The data type.
     verify_shape: Boolean that enables verification of the shape of `value`. If
       `True`, the initializer will throw an error if the shape of `value` is not
       compatible with the shape of the initialized tensor.
 
+  Raises:
+    TypeError: If the input `value` is not one of the expected types.
+
   Examples:
     The following example can be rewritten using a numpy.ndarray instead
     of the `value` list, even reshaped, as shown in the two commented lines
@@ -151,17 +199,33 @@ class Constant(Initializer):
   """
 
   def __init__(self, value=0, dtype=dtypes.float32, verify_shape=False):
-    self.value = value
-    self.dtype = dtype
-    self.verify_shape = verify_shape
+    if not (np.isscalar(value) or isinstance(value, (list, tuple, np.ndarray))):
+      raise TypeError(
+          "Invalid type for initial value: %s (expected Python scalar, list or "
+          "tuple of values, or numpy.ndarray)." % type(value))
 
-  def __call__(self, shape, dtype=None, partition_info=None):
+    self.value = value
+    self.dtype = dtypes.as_dtype(dtype)
+    self._verify_shape = verify_shape
+
+  def __call__(self, shape, dtype=None, partition_info=None, verify_shape=None):
     if dtype is None:
       dtype = self.dtype
-    return constant_op.constant(self.value, dtype=dtype, shape=shape,
-                                verify_shape=self.verify_shape)
+    if verify_shape is None:
+      verify_shape = self._verify_shape
+    return constant_op.constant(
+        self.value, dtype=dtype, shape=shape, verify_shape=verify_shape)
+
+  def get_config(self):
+    # We don't include `verify_shape` for compatibility with Keras.
+    # `verify_shape` should be passed as an argument to `__call__` rather
+    # than as a constructor argument: conceptually it isn't a property
+    # of the initializer.
+    return {"value": self.value, "dtype": self.dtype.name}
 
 
+@tf_export("keras.initializers.RandomUniform", "initializers.random_uniform",
+           "random_uniform_initializer")
 class RandomUniform(Initializer):
   """Initializer that generates tensors with a uniform distribution.
 
@@ -180,15 +244,25 @@ class RandomUniform(Initializer):
     self.minval = minval
     self.maxval = maxval
     self.seed = seed
-    self.dtype = dtype
+    self.dtype = dtypes.as_dtype(dtype)
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
       dtype = self.dtype
-    return random_ops.random_uniform(shape, self.minval, self.maxval,
-                                     dtype, seed=self.seed)
+    return random_ops.random_uniform(
+        shape, self.minval, self.maxval, dtype, seed=self.seed)
+
+  def get_config(self):
+    return {
+        "minval": self.minval,
+        "maxval": self.maxval,
+        "seed": self.seed,
+        "dtype": self.dtype.name
+    }
 
 
+@tf_export("keras.initializers.RandomNormal", "initializers.random_normal",
+           "random_normal_initializer")
 class RandomNormal(Initializer):
   """Initializer that generates tensors with a normal distribution.
 
@@ -207,15 +281,25 @@ class RandomNormal(Initializer):
     self.mean = mean
     self.stddev = stddev
     self.seed = seed
-    self.dtype = _assert_float_dtype(dtype)
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
       dtype = self.dtype
-    return random_ops.random_normal(shape, self.mean, self.stddev,
-                                    dtype, seed=self.seed)
+    return random_ops.random_normal(
+        shape, self.mean, self.stddev, dtype, seed=self.seed)
+
+  def get_config(self):
+    return {
+        "mean": self.mean,
+        "stddev": self.stddev,
+        "seed": self.seed,
+        "dtype": self.dtype.name
+    }
 
 
+@tf_export("keras.initializers.TruncatedNormal",
+           "initializers.truncated_normal", "truncated_normal_initializer")
 class TruncatedNormal(Initializer):
   """Initializer that generates a truncated normal distribution.
 
@@ -239,15 +323,25 @@ class TruncatedNormal(Initializer):
     self.mean = mean
     self.stddev = stddev
     self.seed = seed
-    self.dtype = _assert_float_dtype(dtype)
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
       dtype = self.dtype
-    return random_ops.truncated_normal(shape, self.mean, self.stddev,
-                                       dtype, seed=self.seed)
+    return random_ops.truncated_normal(
+        shape, self.mean, self.stddev, dtype, seed=self.seed)
+
+  def get_config(self):
+    return {
+        "mean": self.mean,
+        "stddev": self.stddev,
+        "seed": self.seed,
+        "dtype": self.dtype.name
+    }
 
 
+@tf_export("initializers.uniform_unit_scaling",
+           "uniform_unit_scaling_initializer")
 class UniformUnitScaling(Initializer):
   """Initializer that generates tensors without scaling variance.
 
@@ -275,10 +369,13 @@ class UniformUnitScaling(Initializer):
     dtype: The data type. Only floating point types are supported.
   """
 
+  @deprecated(None,
+              "Use tf.initializers.variance_scaling instead with distribution="
+              "uniform to get equivalent behavior.")
   def __init__(self, factor=1.0, seed=None, dtype=dtypes.float32):
     self.factor = factor
     self.seed = seed
-    self.dtype = _assert_float_dtype(dtype)
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
@@ -296,10 +393,15 @@ class UniformUnitScaling(Initializer):
     # Avoid errors when initializing zero-size tensors.
     input_size = max(input_size, 1.0)
     max_val = math.sqrt(3 / input_size) * self.factor
-    return random_ops.random_uniform(shape, -max_val, max_val,
-                                     dtype, seed=self.seed)
+    return random_ops.random_uniform(
+        shape, -max_val, max_val, dtype, seed=self.seed)
+
+  def get_config(self):
+    return {"factor": self.factor, "seed": self.seed, "dtype": self.dtype.name}
 
 
+@tf_export("keras.initializers.VarianceScaling",
+           "initializers.variance_scaling", "variance_scaling_initializer")
 class VarianceScaling(Initializer):
   """Initializer capable of adapting its scale to the shape of weights tensors.
 
@@ -313,7 +415,7 @@ class VarianceScaling(Initializer):
   With `distribution="uniform"`, samples are drawn from a uniform distribution
   within [-limit, limit], with `limit = sqrt(3 * scale / n)`.
 
-  Arguments:
+  Args:
     scale: Scaling factor (positive float).
     mode: One of "fan_in", "fan_out", "fan_avg".
     distribution: Random distribution to use. One of "normal", "uniform".
@@ -327,7 +429,8 @@ class VarianceScaling(Initializer):
       "distribution" arguments.
   """
 
-  def __init__(self, scale=1.0,
+  def __init__(self,
+               scale=1.0,
                mode="fan_in",
                distribution="normal",
                seed=None,
@@ -343,7 +446,7 @@ class VarianceScaling(Initializer):
     self.mode = mode
     self.distribution = distribution
     self.seed = seed
-    self.dtype = _assert_float_dtype(dtype)
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
 
   def __call__(self, shape, dtype=None, partition_info=None):
     if dtype is None:
@@ -361,20 +464,33 @@ class VarianceScaling(Initializer):
       scale /= max(1., (fan_in + fan_out) / 2.)
     if self.distribution == "normal":
       stddev = math.sqrt(scale)
-      return random_ops.truncated_normal(shape, 0.0, stddev,
-                                         dtype, seed=self.seed)
+      return random_ops.truncated_normal(
+          shape, 0.0, stddev, dtype, seed=self.seed)
     else:
       limit = math.sqrt(3.0 * scale)
-      return random_ops.random_uniform(shape, -limit, limit,
-                                       dtype, seed=self.seed)
+      return random_ops.random_uniform(
+          shape, -limit, limit, dtype, seed=self.seed)
+
+  def get_config(self):
+    return {
+        "scale": self.scale,
+        "mode": self.mode,
+        "distribution": self.distribution,
+        "seed": self.seed,
+        "dtype": self.dtype.name
+    }
 
 
+@tf_export("keras.initializers.Orthogonal", "initializers.orthogonal",
+           "orthogonal_initializer")
 class Orthogonal(Initializer):
   """Initializer that generates an orthogonal matrix.
 
-  If the shape of the tensor to initialize is two-dimensional, i is initialized
-  with an orthogonal matrix obtained from the singular value decomposition of a
-  matrix of uniform random numbers.
+  If the shape of the tensor to initialize is two-dimensional, it is initialized
+  with an orthogonal matrix obtained from the QR decomposition of a matrix of
+  uniform random numbers. If the matrix has fewer rows than columns then the
+  output will have orthogonal rows. Otherwise, the output will have orthogonal
+  columns.
 
   If the shape of the tensor to initialize is more than two-dimensional,
   a matrix of shape `(shape[0] * ... * shape[n - 2], shape[n - 1])`
@@ -389,9 +505,9 @@ class Orthogonal(Initializer):
       for behavior.
   """
 
-  def __init__(self, gain=1.0, dtype=dtypes.float32, seed=None):
+  def __init__(self, gain=1.0, seed=None, dtype=dtypes.float32):
     self.gain = gain
-    self.dtype = _assert_float_dtype(dtype)
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
     self.seed = seed
 
   def __call__(self, shape, dtype=None, partition_info=None):
@@ -407,21 +523,55 @@ class Orthogonal(Initializer):
     for dim in shape[:-1]:
       num_rows *= dim
     num_cols = shape[-1]
-    flat_shape = (num_rows, num_cols)
+    flat_shape = (num_cols, num_rows) if num_rows < num_cols else (num_rows,
+                                                                   num_cols)
 
     # Generate a random matrix
-    a = random_ops.random_uniform(flat_shape, dtype=dtype, seed=self.seed)
-    # Compute the svd
-    _, u, v = linalg_ops.svd(a, full_matrices=False)
-    # Pick the appropriate singular value decomposition
-    if num_rows > num_cols:
-      q = u
-    else:
-      # Tensorflow departs from numpy conventions
-      # such that we need to transpose axes here
-      q = array_ops.transpose(v)
+    a = random_ops.random_normal(flat_shape, dtype=dtype, seed=self.seed)
+    # Compute the qr factorization
+    q, r = linalg_ops.qr(a, full_matrices=False)
+    # Make Q uniform
+    d = array_ops.diag_part(r)
+    ph = d / math_ops.abs(d)
+    q *= ph
+    if num_rows < num_cols:
+      q = array_ops.matrix_transpose(q)
     return self.gain * array_ops.reshape(q, shape)
 
+  def get_config(self):
+    return {"gain": self.gain, "seed": self.seed, "dtype": self.dtype.name}
+
+
+@tf_export("keras.initializers.Identity", "initializers.identity")
+class Identity(Initializer):
+  """Initializer that generates the identity matrix.
+
+  Only use for 2D matrices.
+
+  Args:
+    gain: Multiplicative factor to apply to the identity matrix.
+    dtype: The type of the output.
+  """
+
+  def __init__(self, gain=1.0, dtype=dtypes.float32):
+    self.gain = gain
+    self.dtype = _assert_float_dtype(dtypes.as_dtype(dtype))
+
+  def __call__(self, shape, dtype=None, partition_info=None):
+    full_shape = shape if partition_info is None else partition_info.full_shape
+    if len(full_shape) != 2:
+      raise ValueError(
+          "Identity matrix initializer can only be used for 2D matrices.")
+    if dtype is None:
+      dtype = self.dtype
+    initializer = linalg_ops.eye(*full_shape, dtype=dtype)
+    if partition_info is not None:
+      initializer = array_ops.slice(initializer, partition_info.var_offset,
+                                    shape)
+    return self.gain * initializer
+
+  def get_config(self):
+    return {"gain": self.gain, "dtype": self.dtype.name}
 
 # Aliases.
 
@@ -435,9 +585,12 @@ truncated_normal_initializer = TruncatedNormal
 uniform_unit_scaling_initializer = UniformUnitScaling
 variance_scaling_initializer = VarianceScaling
 orthogonal_initializer = Orthogonal
+identity_initializer = Identity
+
 # pylint: enable=invalid-name
 
 
+@tf_export("glorot_uniform_initializer")
 def glorot_uniform_initializer(seed=None, dtype=dtypes.float32):
   """The Glorot uniform initializer, also called Xavier uniform initializer.
 
@@ -448,7 +601,7 @@ def glorot_uniform_initializer(seed=None, dtype=dtypes.float32):
 
   Reference: http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf
 
-  Arguments:
+  Args:
     seed: A Python integer. Used to create random seeds. See
       @{tf.set_random_seed}
       for behavior.
@@ -457,13 +610,11 @@ def glorot_uniform_initializer(seed=None, dtype=dtypes.float32):
   Returns:
     An initializer.
   """
-  return variance_scaling_initializer(scale=1.0,
-                                      mode="fan_avg",
-                                      distribution="uniform",
-                                      seed=seed,
-                                      dtype=dtype)
+  return variance_scaling_initializer(
+      scale=1.0, mode="fan_avg", distribution="uniform", seed=seed, dtype=dtype)
 
 
+@tf_export("glorot_normal_initializer")
 def glorot_normal_initializer(seed=None, dtype=dtypes.float32):
   """The Glorot normal initializer, also called Xavier normal initializer.
 
@@ -474,7 +625,7 @@ def glorot_normal_initializer(seed=None, dtype=dtypes.float32):
 
   Reference: http://jmlr.org/proceedings/papers/v9/glorot10a/glorot10a.pdf
 
-  Arguments:
+  Args:
     seed: A Python integer. Used to create random seeds. See
       @{tf.set_random_seed}
       for behavior.
@@ -483,11 +634,8 @@ def glorot_normal_initializer(seed=None, dtype=dtypes.float32):
   Returns:
     An initializer.
   """
-  return variance_scaling_initializer(scale=1.0,
-                                      mode="fan_avg",
-                                      distribution="normal",
-                                      seed=seed,
-                                      dtype=dtype)
+  return variance_scaling_initializer(
+      scale=1.0, mode="fan_avg", distribution="normal", seed=seed, dtype=dtype)
 
 
 # Utility functions.
@@ -496,7 +644,7 @@ def glorot_normal_initializer(seed=None, dtype=dtypes.float32):
 def _compute_fans(shape):
   """Computes the number of input and output units for a weight shape.
 
-  Arguments:
+  Args:
     shape: Integer shape tuple or TF tensor shape.
 
   Returns:
